package eu.eudat.security;

import java.io.IOException;
import java.security.GeneralSecurityException;

import eu.eudat.models.login.LoginInfo;
import eu.eudat.models.security.Principal;
import eu.eudat.security.validators.TokenValidatorFactory;
import eu.eudat.security.validators.TokenValidatorFactoryImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.authentication.AuthenticationServiceException;
import org.springframework.security.core.AuthenticationException;
import org.springframework.stereotype.Component;

import eu.eudat.dao.entities.UserInfoDao;
import eu.eudat.exceptions.NonValidTokenException;
import eu.eudat.security.validators.google.GoogleTokenValidator;

@Component
public class CustomAuthenticationProvider {


	@Autowired private TokenValidatorFactory tokenValidatorFactory;

	public Principal authenticate(LoginInfo credentials) throws AuthenticationException, GeneralSecurityException {
			String token = credentials.getTicket();
			try {
				Principal principal = this.tokenValidatorFactory.getProvider(credentials.getProvider()).validateToken(credentials);
				return  principal;
			} catch (NonValidTokenException e) {
				e.printStackTrace();
				System.out.println("Could not validate a user by his token! Reason: " + e.getMessage());
				throw new AuthenticationServiceException("Token validation failed - Not a valid token");
			} catch (IOException e) {
				e.printStackTrace();
				throw new AuthenticationServiceException("IO Exeption");
			}
	}
}