import { InvitationComponent } from '../invitation/invitation.component';
import { Component, ViewChild, OnInit, AfterViewInit } from "@angular/core";
import { MatPaginator, MatSort, MatSnackBar, MatDialog } from "@angular/material";
import { Router } from "@angular/router";
import { TranslateService } from "@ngx-translate/core";
import { DataSource } from "@angular/cdk/table";
import { Observable } from "rxjs/Observable";
import { SnackBarNotificationComponent } from "../shared/components/notificaiton/snack-bar-notification.component";
import { DataManagementPlanService } from "../services/data-management-plan/data-management-plan.service";
import { DataManagementPlanCriteriaComponent } from "../shared/components/criteria/data-management-plan/dmp-criteria.component";
import { DataManagementPlanCriteria } from "../models/criteria/data-management-plan/DataManagementPlanCriteria";
import { DataManagementPlanListingModel } from "../models/data-managemnt-plans/DataManagementPlanListingModel";
import { DataTableRequest } from '../models/data-table/DataTableRequest';



@Component({
  selector: 'app-dmp-listing-component',
  templateUrl: 'dmp-listing.component.html',
  styleUrls: ['./dmp-listing.component.scss'],
  providers: [DataManagementPlanService]
})
export class DataManagementPlanListingComponent implements OnInit {

  @ViewChild(MatPaginator) _paginator: MatPaginator;
  @ViewChild(MatSort) sort: MatSort;
  @ViewChild(DataManagementPlanCriteriaComponent) criteria: DataManagementPlanCriteriaComponent;

  dataSource: DataManagementPlanDataSource | null;
  displayedColumns: String[] = ['name', 'project', 'profile', 'researchers', 'organisations', 'version', 'actions'];

  constructor(
    private dataManagementPlanService: DataManagementPlanService,
    private router: Router,
    private languageService: TranslateService,
    public snackBar: MatSnackBar,
    public dialog: MatDialog
  ) {

  }

  ngOnInit() {
    this.criteria.setCriteria(this.getDefaultCriteria());
    this.refresh();
    this.criteria.setRefreshCallback(() => this.refresh());
  }

  refresh() {
    this.dataSource = new DataManagementPlanDataSource(this.dataManagementPlanService, this._paginator, this.sort, this.languageService, this.snackBar, this.criteria, );
  }

  rowClick(rowId: String) {
    this.router.navigate(['/dmp/' + rowId]);
  }

  getDefaultCriteria(): DataManagementPlanCriteria {
    const defaultCriteria = new DataManagementPlanCriteria();
    return defaultCriteria;
  }

  openShareDialog(rowId: any, rowName: any) {
    let dialogRef = this.dialog.open(InvitationComponent, {
      height: '200px',
      width: '700px',
      data: {
        dmpId: rowId,
        dmpName: rowName
      }
    });
  }
}

export class DataManagementPlanDataSource extends DataSource<DataManagementPlanListingModel> {

  totalCount = 0;
  isLoadingResults = false;

  constructor(
    private _service: DataManagementPlanService,
    private _paginator: MatPaginator,
    private _sort: MatSort,
    private _languageService: TranslateService,
    private _snackBar: MatSnackBar,
    private _criteria: DataManagementPlanCriteriaComponent
  ) {
    super();
  }

  connect(): Observable<DataManagementPlanListingModel[]> {
    const displayDataChanges = [
      this._paginator.page
    ];

    return Observable.merge(...displayDataChanges)
      .startWith(null)
      .switchMap(() => {
        setTimeout(() => {
          this.isLoadingResults = true;
        });
        const startIndex = this._paginator.pageIndex * this._paginator.pageSize;
        const request = new DataTableRequest<DataManagementPlanCriteria>(startIndex, this._paginator.pageSize);
        request.criteria = this._criteria.criteria;
        return this._service.getPaged(request);
      })
      .catch((error: any) => {
        this._snackBar.openFromComponent(SnackBarNotificationComponent, {
          data: { message: 'GENERAL.SNACK-BAR.FORMS-BAD-REQUEST', language: this._languageService },
          duration: 3000,
          extraClasses: ['snackbar-warning']
        });
        return Observable.of(null);
      })
      .map(result => {
        setTimeout(() => {
          this.isLoadingResults = false;
        });
        return result;
      })
      .map(result => {
        if (!result) { return []; }
        if (this._paginator.pageIndex === 0) { this.totalCount = result.totalCount; }
        return result.data;
      });
  }

  disconnect() {
  }
}
